/**
 * 
 */
package org.gcube.informationsystem.resourceregistry.type;

import java.util.List;

import org.gcube.informationsystem.base.reference.AccessType;
import org.gcube.informationsystem.base.reference.ISManageable;
import org.gcube.informationsystem.base.reference.properties.BaseProperty;
import org.gcube.informationsystem.base.reference.properties.Header;
import org.gcube.informationsystem.base.reference.relations.BaseRelation;
import org.gcube.informationsystem.context.reference.entities.Context;
import org.gcube.informationsystem.model.reference.entities.Entity;
import org.gcube.informationsystem.model.reference.entities.Facet;
import org.gcube.informationsystem.model.reference.entities.Resource;
import org.gcube.informationsystem.model.reference.properties.Encrypted;
import org.gcube.informationsystem.model.reference.properties.Property;
import org.gcube.informationsystem.model.reference.relations.ConsistsOf;
import org.gcube.informationsystem.model.reference.relations.IsRelatedTo;
import org.gcube.informationsystem.model.reference.relations.Relation;
import org.gcube.informationsystem.resourceregistry.api.exceptions.schema.SchemaAlreadyPresentException;
import org.gcube.informationsystem.resourceregistry.instances.type.SchemaManagement;
import org.gcube.informationsystem.resourceregistry.instances.type.SchemaManagementImpl;
import org.gcube.informationsystem.types.TypeBinder;
import org.gcube.informationsystem.types.impl.entities.EntityTypeDefinitionImpl;
import org.gcube.informationsystem.types.impl.properties.PropertyTypeDefinitionImpl;
import org.gcube.informationsystem.types.impl.relations.RelationTypeDefinitionImpl;
import org.gcube.informationsystem.types.reference.TypeDefinition;
import org.gcube.informationsystem.types.reference.entities.EntityTypeDefinition;
import org.gcube.informationsystem.types.reference.properties.PropertyTypeDefinition;
import org.gcube.informationsystem.types.reference.relations.RelationTypeDefinition;
import org.gcube.informationsystem.utils.ISMapper;
import org.gcube.resourcemanagement.model.reference.entities.facets.AccessPointFacet;
import org.gcube.resourcemanagement.model.reference.entities.facets.ContactFacet;
import org.gcube.resourcemanagement.model.reference.entities.resources.Actor;
import org.gcube.resourcemanagement.model.reference.entities.resources.EService;
import org.gcube.resourcemanagement.model.reference.entities.resources.RunningPlugin;
import org.gcube.resourcemanagement.model.reference.properties.ValueSchema;
import org.gcube.resourcemanagement.model.reference.relations.consistsof.HasVolatileMemory;
import org.gcube.resourcemanagement.model.reference.relations.isrelatedto.Hosts;
import org.gcube.resourcemanagement.model.reference.relations.isrelatedto.Uses;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Luca Frosini (ISTI - CNR)
 */
public class SchemaManagementImplTest {

	private static Logger logger = LoggerFactory
			.getLogger(SchemaManagementImplTest.class);

	@Test
	public void registerPropertyTypeSchema() throws Exception {
		Class<? extends BaseProperty> clz = Header.class;
		String json = TypeBinder.serializeType(clz);
		logger.debug(json);
		//new SchemaManagementImpl().create(json, AccessType.PROPERTY);
	}

	@Test
	public void getPropertyTypeSchema() throws Exception {
		String json = new SchemaManagementImpl().read(Property.NAME, false);
		logger.debug(json);
	}

	@Test
	public void registerFacetSchema() throws Exception {
		Class<? extends Facet> clz = ContactFacet.class;
		String json = TypeBinder.serializeType(clz);
		TypeBinder.deserializeTypeDefinition(json);
	}

	@Test
	public void getFacetSchema() throws Exception {
		String json = new SchemaManagementImpl().read(ContactFacet.NAME, false);
		logger.info(json);
		List<TypeDefinition> typeDefinitions = TypeBinder.deserializeTypeDefinitions(json);
		logger.info("{}", typeDefinitions);
		
	}

	@Test
	public void registerEntityTypeSchema() throws Exception {
		Class<? extends Entity> clz = Entity.class;
		String json = TypeBinder.serializeType(clz);
		logger.trace(json);
		// new SchemaManagementImpl().registerEntitySchema(json);
	}

	@Test
	public void registerResourceSchema() throws Exception {
		Class<? extends Resource> clz = Resource.class;
		String json = TypeBinder.serializeType(clz);
		logger.info(json);
		// new SchemaManagementImpl().registerFacetSchema(json);
	}

	@Test
	public void getResourceSchema() throws Exception {
		String json = new SchemaManagementImpl().read(Actor.NAME, false);
		logger.trace(json);
	}

	@Test
	public void registerRelation() throws Exception {
		Class<? extends ISManageable> clz = Hosts.class;
		String json = TypeBinder.serializeType(clz);
		logger.trace(json);
		// new SchemaManagementImpl().registerFacetSchema(json);
	}

	@Test
	public void getList() throws Exception {
		logger.debug("\n\n\n");

		boolean includeSubTypes = true;

		SchemaManagement schemaManagement = new SchemaManagementImpl();
		String list = schemaManagement.read(BaseProperty.NAME, includeSubTypes);
		logger.debug("{} list : {}", BaseProperty.NAME, list);

		List<TypeDefinition> typeDefinitions = ISMapper.unmarshalList(TypeDefinition.class, list);
		logger.debug("{}", typeDefinitions);

		list = schemaManagement.read(Entity.NAME,  includeSubTypes);
		logger.debug("{} list : {}", Entity.NAME, list);

		list = schemaManagement.read(Resource.NAME,  includeSubTypes);
		logger.debug("{} list : {}", Resource.NAME, list);

		list = schemaManagement.read(Facet.NAME,  includeSubTypes);
		logger.debug("{} list : {}", Facet.NAME, list);

		list = schemaManagement.read(Relation.NAME,  includeSubTypes);
		logger.debug("{} list : {}", Relation.NAME, list);

		list = schemaManagement.read(ConsistsOf.NAME,  includeSubTypes);
		logger.debug("{} list : {}", ConsistsOf.NAME, list);

		list = schemaManagement.read(IsRelatedTo.NAME, includeSubTypes);
		logger.debug("{} list : {}", IsRelatedTo.NAME, list);

	}
	
	@Test(expected=SchemaAlreadyPresentException.class)
	public void createPropertyType() throws Exception {
		PropertyTypeDefinition<ValueSchema> propertyTypeDefinition = new PropertyTypeDefinitionImpl<>(ValueSchema.class);
		
		SchemaManagement schemaManagement = new SchemaManagementImpl();
		((SchemaManagementImpl) schemaManagement).setTypeName(ValueSchema.NAME);
		String ret = schemaManagement.create(ISMapper.marshal(propertyTypeDefinition), AccessType.PROPERTY);
		
		logger.debug(ret);
		
	}
	
	@Test(expected=SchemaAlreadyPresentException.class)
	public void createEncryptedType() throws Exception {
		PropertyTypeDefinition<Encrypted> propertyTypeDefinition = new PropertyTypeDefinitionImpl<>(Encrypted.class);
		
		SchemaManagement schemaManagement = new SchemaManagementImpl();
		((SchemaManagementImpl) schemaManagement).setTypeName(Encrypted.NAME);
		String ret = schemaManagement.create(ISMapper.marshal(propertyTypeDefinition), AccessType.PROPERTY);
		
		logger.debug(ret);
		
	}
	
	@Test(expected=SchemaAlreadyPresentException.class)
	public void createContextType() throws Exception {
		EntityTypeDefinition entityTypeDefinition = new EntityTypeDefinitionImpl(Context.class);
		
		SchemaManagement schemaManagement = new SchemaManagementImpl();
		((SchemaManagementImpl) schemaManagement).setTypeName(Context.NAME);
		String ret = schemaManagement.create(ISMapper.marshal(entityTypeDefinition), AccessType.BASE_ENTITY);
		
		logger.debug(ret);

	}
	
	@Test(expected=SchemaAlreadyPresentException.class)
	public void createFacetType() throws Exception {
		EntityTypeDefinition entityTypeDefinition = new EntityTypeDefinitionImpl(AccessPointFacet.class);
		
		SchemaManagement schemaManagement = new SchemaManagementImpl();
		((SchemaManagementImpl) schemaManagement).setTypeName(AccessPointFacet.NAME);
		String ret = schemaManagement.create(ISMapper.marshal(entityTypeDefinition), AccessType.FACET);
		
		logger.debug(ret);
		
	}
	
	@Test(expected=SchemaAlreadyPresentException.class)
	public void createResourceType() throws Exception {
		EntityTypeDefinition entityTypeDefinition = new EntityTypeDefinitionImpl(EService.class);
		
		SchemaManagement schemaManagement = new SchemaManagementImpl();
		((SchemaManagementImpl) schemaManagement).setTypeName(EService.NAME);
		String ret = schemaManagement.create(ISMapper.marshal(entityTypeDefinition), AccessType.RESOURCE);
		
		logger.debug(ret);
		
		entityTypeDefinition = new EntityTypeDefinitionImpl(RunningPlugin.class);
		
		schemaManagement = new SchemaManagementImpl();
		((SchemaManagementImpl) schemaManagement).setTypeName(RunningPlugin.NAME);
		ret = schemaManagement.create(ISMapper.marshal(entityTypeDefinition), AccessType.RESOURCE);
		
		logger.debug(ret);
		
	}
	
	@Test(expected=SchemaAlreadyPresentException.class)
	public void createRelationTypeDefinitionType() throws Exception {
		RelationTypeDefinition relationTypeDefinition = new RelationTypeDefinitionImpl((Class<? extends BaseRelation<?,?>>) RelationTypeDefinition.class);
		
		SchemaManagement schemaManagement = new SchemaManagementImpl();
		((SchemaManagementImpl) schemaManagement).setTypeName(RelationTypeDefinition.NAME);
		String ret = schemaManagement.create(ISMapper.marshal(relationTypeDefinition), AccessType.BASE_RELATION);
		
		logger.debug(ret);
		
	}
	
	@Test(expected=SchemaAlreadyPresentException.class)
	public void createIsRelatedToType() throws Exception {
		@SuppressWarnings("unchecked")
		RelationTypeDefinition relationTypeDefinition = new RelationTypeDefinitionImpl((Class<? extends BaseRelation<?,?>>) Uses.class);
		
		SchemaManagement schemaManagement = new SchemaManagementImpl();
		((SchemaManagementImpl) schemaManagement).setTypeName(Uses.NAME);
		String ret = schemaManagement.create(ISMapper.marshal(relationTypeDefinition), AccessType.IS_RELATED_TO);
		
		logger.debug(ret);
		
	}
	
	@Test(expected=SchemaAlreadyPresentException.class)
	public void createConsistsOfType() throws Exception {
		@SuppressWarnings("unchecked")
		RelationTypeDefinition relationTypeDefinition = new RelationTypeDefinitionImpl((Class<? extends BaseRelation<?,?>>) HasVolatileMemory.class);
		
		SchemaManagement schemaManagement = new SchemaManagementImpl();
		((SchemaManagementImpl) schemaManagement).setTypeName(HasVolatileMemory.NAME);
		String ret = schemaManagement.create(ISMapper.marshal(relationTypeDefinition), AccessType.CONSISTS_OF);
		
		logger.debug(ret);
		
	}
	
	
}
